<#compress>
<#import "../../application/docbook+xml/utils.ftl" as utils>
<#-- Import common macros and functions -->
<#import "../../application/docbook+xml/macros_common_general.ftl" as com>
<#-- Initialize common variables -->
<@com.initializeMainVariables/>	
<#-- Root entity -->
<#assign rootEntity = entity.root>
<#-- Utility list to remember tarversed entities -->
<#assign traveresedEntityKeyList = [rootEntity.documentKey]>
<#-- Print CSV header row -->
<@printCsvHeadersRow />
<#-- Utility list to remember entities yet to be traversed -->
<#assign toTravereseEntityList = []>
<#-- Go throught the found referenced entities and traverse them. NOTE: the list can grow further as entities can reference further entities. Unclear whether this works well in FreeMarker -->
<#-- FreeMarker does not support do-while loops as such do this workaround with maxLoop -->

<#----------------------------------------------->
<#-- Macro and function definitions start here -->
<#----------------------------------------------->

<#-- Maxloop sets the limit at which the component hierarchy is travered -->
<#assign maxLoop=5>
  
  <#-- Traverse the main entity -->
  <@traverseEntity rootEntity />

    <#list 1..maxLoop as x>           
      <#-- Remove documents that have been already traversed -->
      <#assign toTravereseEntityList = utils.removeDocumentWitKey(toTravereseEntityList, traveresedEntityKeyList) >
      <#if !(toTravereseEntityList?has_content)>
        <#-- Stop looping if there is nothing to traverse -->
        <#break>
      </#if>
      <#-- Travers each entity from the list -->
      <#list toTravereseEntityList as entityItem>
        <#assign traveresedEntityKeyList= traveresedEntityKeyList + [entityItem.documentKey]>
          <@traverseEntity entityItem />
      </#list>          
    </#list>

<#macro traverseEntity entityDoc>
  <#if utils.isCompositeEntityDocument(entityDoc)>
    <@traverseCompositeEntity entityDoc />
  <#else>
    <@traverseSimpleEntity entityDoc />
  </#if>
</#macro>

<#macro traverseSimpleEntity entityDoc>
  <@traverseDoc entityDoc entityDoc 0 />
</#macro>

<#macro traverseCompositeEntity entityDoc>
  <#local tableOfContents=iuclid.localizeTreeFor(entityDoc.documentType, entityDoc.submissionType, entityDoc.documentKey, utils.getInheritedTemplates(entityDoc), utils.getRelatedCategories(entityDoc), utils.getRelatedMixtures(entityDoc))>
  <@traverseToc tableOfContents entityDoc 0 />
</#macro>

<#macro traverseToc sectionNode entityDocument level=0>
  <#if sectionNode?has_content>
    <#-- iterate through the section documents under this section if there are any -->
    <#if sectionNode.content?has_content>
      <#list sectionNode.content as doc>
        <#-- traverse through the document's blocks and fields aka elements -->
        <@traverseDoc doc entityDocument level+1 sectionNode />
      </#list>
    </#if>
    <#-- iterate through the child sections of the node and do a recursive call -->
    <#if sectionNode?children?has_content>
      <#list sectionNode?children as child> 
        <@traverseToc child entityDocument level+1 />    
      </#list>
    </#if>
  </#if>
</#macro>

<#macro traverseDoc docElementNode entityDocument level=0 sectionNode="" isDocument=true>
  <#if docElementNode?has_content>
    <#-- Only document level -->
    <#if isDocument>
      <@printInboundReferencesForDocument docElementNode entityDocument sectionNode />
    </#if>
    <#-- Check if the element node is a reference field (single or multiple) and remember referenced entities for later traversal in toTravereseEntityList -->
    <#assign toTravereseEntityList = utils.checkAndRememberReferencedEntity(docElementNode, toTravereseEntityList, traveresedEntityKeyList) />
    <#-- iterate through the child structure elements of this document element if there are any and do a recursive call -->
    <#if docElementNode?children?has_content>
      <#list docElementNode?children as child>
        <@traverseDoc child entityDocument level+1 sectionNode false/>
      </#list>
    </#if>
  </#if>
</#macro>

<#macro printInboundReferencesForDocument sectionDoc entityDocument sectionNode>

  <#assign sectionNameForSourceDoc><@sectionNodeDetails sectionNode/></#assign>

  <#-- Section doc type -->
  <#local sectionDocType = sectionDoc.documentType>

  <#-- Section sub doc type -->
  <#local sectionDocSubType = sectionDoc.documentSubType>

  <#if !utils.isEntityDocument(sectionDoc)>
    <#local sectionDocType = sectionDocType + "." + sectionDoc.documentSubType>
  </#if>

  <#-- The web URL pointing to the document -->
  <#local docUrl=utils.getDocumentUrl(sectionDoc) />

   <#list com.inboundReferences(sectionDoc.documentKey) as inboundReferenceKey>
        
        <#if inboundReferenceKey?has_content>
        <#assign inboundReference = iuclid.getDocumentForKey(inboundReferenceKey) />
            <#if inboundReference?has_content>
            
                <#local docUrl=utils.getDocumentUrl(sectionDoc) />
                <#local docUrlInboundReference=utils.getDocumentUrl(inboundReference) />
                <@iuclid.label for=inboundReference var="referenceFieldLabel"/>	

                    <@tableOutput inboundReference referenceFieldLabel entityDocument sectionNode sectionNameForSourceDoc sectionDoc docUrl docUrlInboundReference />
            </#if>
        </#if>
    </#list>
  

</#macro>

<#macro sectionNodeDetails sectionNode>
  <#if sectionNode?has_content>
   <@com.text sectionNode.number /> <@com.text sectionNode.title />
  </#if>
</#macro>

<#macro tableOutput reference referenceFieldLabel entityDocument sectionNode sectionNameForDocument sectionDoc docUrl docUrlInboundReference>
<#compress>
<@utils.csvValue entityDocument.name/>,<@utils.csvValue entityDocument.documentType />,<@utils.csvValue sectionNameForDocument />,<@utils.csvValue sectionDoc.documentType/>,<@utils.csvValue sectionDoc.name />,${docUrl},<@utils.csvValue referenceFieldLabel />,<@utils.csvValue reference.documentType/>,<@utils.csvValue reference.name />,${docUrlInboundReference},<#if isCrossReference(reference, sectionDoc)>Yes - Cross-reference made from an Endpoint Study Record<#else>No</#if>
</#compress>
</#macro>

<#function isCrossReference inboundReference sectionDoc>

<#if inboundReference.documentType=="ENDPOINT_STUDY_RECORD">
      <#return true>
        <#else>
        <#return false>
</#if>
<#return false>
</#function>

<#macro printCsvHeadersRow>
"Main entity name","Main entity Type","Section name and number","Document Type","Document Name","Document url","Inbound section document name","Inbound entity type","Inbound document name","Inbound document url","is Cross-reference made from an Endpoint Study Record?"
</#macro>
</#compress>
